# Mashroom Offer

The Offer service is intended to manage real estate renting offers between tenants and landlords.  

## GRPC API

Method | Scope | Description
-|-|-
`Get` | | Finds, validates (depending on the request), and returns an offer by ID and user
`ListByProperty` | | Finds and returns all offers for the specified property
`ListByPropertyTenant` | | Finds and returns all offers for the specified property and tenant
`ListByPropertyLandlord` | | Finds and returns all offers for the specified property and landlord (except PENDING, CANCELED offers)
`ListOutgoing` | | Finds and returns all offers for the specified tenant, property (if specified) and offer status (default filter: PENDING, ACTIVE, SIGNED)
`ListIncoming` | | Finds and returns all offers for the specified property, or (if no property was specified) for the specified landlord, and offer status (default filter: ACTIVE, SIGNING)
`Create` | | Creates a new PENDING offer (if there wasn't any for the specified property) and start a new instance of the "property-offer-process" Workflow
`CreateForRenew` | | Creates a new AWAITING_RENEW offer and sends a notification on all channels
`Submit` | | Sets the status of an existing offer to ACTIVE (if there were no other errors) and sends an "offer-submitted" Workflow message
`Invite` | | Creates a new invite and invitation (via the Invitation service) for the specified offer and user (as inviter)
`UpdateTenant` | | Returns ErrNoAccess
`RemoveTenant` | | Returns ErrNoAccess
`Update` | | Updates an existing offer
`RequestChange` | | Sets the status of an existing offer from ACTIVE to PENDING and sends an "offer-change-requested" Workflow message
`Accept` | | Sets the status of an existing offer to ACCEPTED and then to HOLDING_DEPOSIT (if there were no other errors), sends an "offer-accepted" Workflow message, removes all invites for that offer, and sends notifications to the tenant and landlord
`Cancel` | | Sets the status of an existing offer from ACTIVE or PENDING to CANCELED, sends (if the offer status was ACTIVE) an "offer-ammended" Workflow message, sends an "offer-canceled" Workflow message, and removes all invites for that offer
`Deny` | | Sets the status of an existing offer to DENIED (if there were no other errors), sends an "offer-denied" Workflow message, removes all invites for that offer, and sends various notifications (depending on the current state of the offer)
`Check` | | Sets the status of an existing offer (depending on the "force skip" bool flag in the request); If "force skip" is FALSE: from DEPOSIT_CONFIRMED, AWAITING_CHECKS to AWAITING_APPLICANT_CHECK_PAYMENT (also sends an "offer-run-checks" Workflow message); If "force skip" is TRUE: from DEPOSIT_CONFIRMED, AWAITING_APPLICANT_CHECK_PAYMENT, AWAITING_CHECKS to CONTRACT_REVIEW_AND_CLAUSES (also: cancels invoice and sends a "proceed-without-checks" Workflow message if the status was AWAITING_APPLICANT_CHECK_PAYMENT; sends an "offer-skip-checks" Workflow message if the status was DEPOSIT_CONFIRMED or AWAITING_APPLICANT_CHECK_PAYMENT; sends a "checks-complete" Workflow message if the status was AWAITING_CHECKS)
`RemoveSecondLandlord` | | Clears landlords collection of an existing offer
`ListNotNotified` | | Returns a list of offers (filtered by the specified statuses) for which no scheduled notifications were marked as "notified"
`MarkAsNotified` | | Marks scheduled notifications for specified offer and event types as "notified"
`RequestRequiredGuarantor` | | Sets the status of an existing AWAITING_CHECKS offer to AWAITING_GUARANTOR and sends an "guarantor-required" Workflow message  (Used to make a request to add a new guarantor to the current tenant, if tenant has HIGH risk from rentprofile checking, and the landlord decided to check a new guarantor of the failed tenant; also can be called if the tenant and his guarantor still have HIGH risks)
`AddGuarantors` | | Adds new guarantors to the specified AWAITING_GUARANTOR offer (the specified user must be a tenant), sets the offer status to the AWAITING_APPLICANT_CHECK_PAYMENT and sends a "guarantor-added" Workflow message
`ServicesSelect` | | Creates a new invoice (via the Invoice service) for the specified extra services for the specified offer and user (if the offer is in the SIGNED status)
`DepositSelect` | | Returns the parameters of the deposit payment schedule and invoice ID (via the Invoice service)
`ServicesList` | | Returns an extra services list (via the Extras service) for the specified offer and user (if the offer is in the SIGNED status)
`Payment` | | Creates a new payment of the specified type (via the Invoice service) for an existing invoice
`StandingOrder` | | Creates a new standing order payment (via the Invoice service) for an existing invoice
`DirectDebit` | | Creates a new direct debit payment (via the Invoice service)
`DeleteDocument` | | Does nothing
`ListFilter` | | Returns a filtered list of offers
`ServiceGet` | Internal | Finds and returns an offer by ID
`ServiceUpdate` | Internal | Updates an existing offer and sends (depending on the request) a "contract-prepared" Workflow message
`ServiceApprove` | Internal | Approves an existing offer
`ServicePropertyList` | Internal | Finds and returns all "open" offers for the specified property that have tenants
`DeleteOffersByProperty` | | Destroys offers for specified property if the property doesn't exist in the Property service
`DeleteOfferById` | | Marks an existing offer as "deleted"
`RestoreOfferById` | | Unmarks an existing offer as "deleted"
`RenderCsv` | | Creates a CSV file (via the Document service) with various data for the specified offer if the deposit type in the offer is DEPOSIT_REPLACEMENT, and sends (depending on the request) a notification email to orders@mashroom.com
`ExecuteCheck` | | Performs tenants and guarantors identity check (via the Identity service) (This API method is used in the admin tool)
`ServiceRelatedProperties` | Internal | Returns a list of property IDs for which the specified user is a tenant 
`TenantCheckUpdate` | | Updates the specified fields of the tenant collection for the specified offer (This API method is used by the Identity service to update tenant check results)
`AddContractClause` | | Adds a new clause to the specified CONTRACT_REVIEW_AND_CLAUSES offer (the specified user must be a landlord)
`UpdateContractClauses` | | Updates/removes clauses in the specified CONTRACT_REVIEW_AND_CLAUSES offer (the specified user must be a landlord)
`ChangeStateContractClauses` | | Changes the state of the specified clause (accepts/denies the clause) in the specified CONTRACT_REVIEW_AND_CLAUSES offer (the specified user must be a tenant)
`ProgressToSigning` | | Checks if there are any clauses in the PENDING status for the specified CONTRACT_REVIEW_AND_CLAUSES offer, and if there are none, creates a contract (using the Document service) and sends a "contract-prepared" Workflow message
`RenewTenancy` | | Creates a new offer based on the specified one and starts a "property-offer-process" Workflow instance
`AcceptUpdate` | | Deprecated?
`UpdateDraftContract` | | Regenerates (via the Contract service) a draft contract for an existing CONTRACT_REVIEW_AND_CLAUSES offer (This API method is used in the admin tool)
`SkipReferencingStage` | | Sets the status of an existing offer from AWAITING_APPLICANT_CHECK_PAYMENT / AWAITING_CHECKS to CONTRACT_REVIEW_AND_CLAUSES, then: cancels invoice (via the Invoice service) and sends a "proceed-without-checks" Workflow message if the offer was in the AWAITING_APPLICANT_CHECK_PAYMENT status, or just sends a "checks-complete" Workflow message (This API method is used to skip the offer referencing stage using the admin tool)
`ScheduledPaymentList` | | Returns a filtered list of scheduled payments for the specified offer and user
`FilterScheduledPayments` | | Returns a filtered list of all scheduled payments
`ScheduledPaymentAdvanceCreate` | | Updates the specified scheduled payments as to be paid in advance (requests pay in advance)
`ScheduledPaymentAdvanceCancel` | | Cancels pay in advance requests for the specified scheduled payment
`ScheduledPaymentSave` | | Updates the specified scheduled payment (This API method is used in the admin tool to change payout status, pay in advance request confirmation)
`ScheduledPaymentPay` | | Deprecated
`FilterScheduledDeposites` | | Returns a filtered list of scheduled deposits 
`ScheduledDepositUpdate` | | Updates an existing scheduled deposit
`ChangeLogChangeUserName` | | Logs tenant, landlord or guarantor name change in the offer changelog 
`ChangeLogAdd` | | Creates a new entry in the offer changelog
`Payout` | | Creates a new payout to the landlord (as payment via the Invoice service) for an existing invoice and updates the scheduled payment for the specified offer

(TODO: a more detailed description of the methods is needed)

## GRPC coupling

Service | Use
-|-
Account | User account store (only getting data)
Contract | Contracts (legal documents) store
Document | Contract files store
Extras | Extra services (additional paid services) store (only getting data)
Invitation | Invitation creation from the `Invite` API method
Identity | Tenant identity check
Invoice | Invoices store
Notification | Notification by email, platform messaging and sms
Property | Property (real estate) store
Rightmove | [Rightmove](https://www.rightmove.co.uk/) gateway
Signature | Contract signing
Schedule | Used to start a "tenancy-post-termination" Workflow instance at the scheduled time
Zoopla | [Zoopla](https://www.zoopla.co.uk/) gateway

## Databases

DBMS | Extensions
-|-
MongoDB | none
  
Access abstraction: none.  
Migrations: versioned migrations (hardcoded, not embedded) are applied at the service startup.  

Database models are converted to GRPC data structures using special "render" functions.

## Zeebe integration

### Job handler ("offer-service")
Operation | Description | Input | Output
-|-|-|-
`set-status` | Sets/changes the status (state) of the offer to the one specified in the request | id - offer identifier <br /> status - offer status (values: see "Values" tab) | error(optional) – error details
`notify-applicants` | Sends a platform notification to all tenants in the offer except the one specified in the request | id - offer identifier <br /> body - notification text <br /> contract (optional) – contract identifier <br /> tenant (optional) – tenant identifier | error(optional) – error details
`create-contract` | Finds an offer by ID, then finds its property and landlord user account; returns offer's contract if there were no errors |  id - offer identifier | error(optional) – error details
`update-contract` | Sets an offer status to SIGNED and sends an email notification if signatures in the request are ok, there were no errors and IsComplete flag in the request is set; sets the offer status to ERROR otherwise|event - event description <br /> -event_type - event type attribute <br /> signature_request - data about event signature  requests <br />-is_complete - attribute marked contract has been completed <br />-is_declined - attribute marked contract has been declined<br />-has_error -  attribute marked contract has error<br />-metadata - signatures metadata<br />--property_id - property identifier<br />--contract_id - contract identifie<br />--offer_id - order identifier<br />--viewing_id - virew identifier<br />-signatures - signatures data<br />--signer_email_address - signer email<br />--status_code - status of signature(SIGNED ,UNKNOWN_STATUS,AWAITING_SIGNATURE,DECLINED) | waserr(optional) – error details 
`execute-checks` | Performs tenants and guarantors identity check (via the Identity service) | id - check identifier <br /> tenants - tenants emails | error(optional) – error details
`tenant-check-update` | Does the same as `TenantCheckUpdate` but updates fewer fields | id - tenant id <br /> check - check type (DEPOSIT_CONFIRMED, AWAITING_CHECKS to AWAITING_APPLICANT_CHECK_PAYMENT) <br /> data - data about score <br /> -risk_score - valse risk score <br /> -pdf_file_id - pdf file identifier | error(optional) – error details


### Monitoring API

Method | Description
-|-
`Ping` | Returns a successful response
`Start` | Returns a successful response
`Incident` | Returns a successful response
`Finish` | Returns a successful response

## Observability

N/A
