package impl

import (
	"context"
	"time"

	"gitlab.mashroom.com/mashroom/backend/common/generic/log"
	offer "gitlab.mashroom.com/mashroom/backend/offer/api"
	"gitlab.mashroom.com/mashroom/backend/offer/model"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/primitive"
	"go.mongodb.org/mongo-driver/mongo/options"
)

func (server *serverImpl) ChangeLogChangeUserName(
	ctx context.Context,
	request *offer.ChangelogUserNameRequest,
) (*offer.ChangelogGenericResponse, error) {
	var q primitive.M
	changeLog := model.OfferChangelog{
		CreatedAt: time.Time{},
		Initiator: request.Initiator,
		Old:       request.OldName,
		New:       request.NewName,
	}

	switch request.UserType {
	case offer.ChangeLogUserType_LANDLORD:
		q = bson.M{"landlord": request.User}
		changeLog.Action = offer.OfferAction_LANDLORD_NAME_CHANGED
	case offer.ChangeLogUserType_TENANT:
		q = bson.M{"tenants": bson.M{"user": request.User}}
		changeLog.Action = offer.OfferAction_TENANT_NAME_CHANGED
	case offer.ChangeLogUserType_GUARANTOR:
		q = bson.M{"tenants": bson.M{"guarantor": bson.M{"user": request.User}}}
		changeLog.Action = offer.OfferAction_GUARANTOR_NAME_CHANGED
	}

	cur, err := model.Offers.Find(ctx, q, options.Find().SetProjection(bson.D{{Key: "_id", Value: 1}}))
	if err != nil {
		return &offer.ChangelogGenericResponse{Success: false}, err
	}

	for cur.Next(ctx) {
		var ofr model.OfferModel
		if err := cur.Decode(&ofr); err != nil {
			log.Error("Decode error: ", err)
		}
		changeLog.Offer = ofr.ID.String()
		// TODO: Bulk save
		_, err = model.OfferLogs.InsertOne(ctx, &changeLog)
		if err != nil {
			log.Error("OfferLog save error: ", err)
		}
	}
	return &offer.ChangelogGenericResponse{Success: true}, nil
}

func (server *serverImpl) ChangeLogAdd(
	ctx context.Context,
	request *offer.ChangelogRequest,
) (*offer.ChangelogGenericResponse, error) {
	_, err := model.OfferLogs.InsertOne(ctx, request)
	if err != nil {
		return &offer.ChangelogGenericResponse{
			Success: false,
		}, err
	}
	return &offer.ChangelogGenericResponse{
		Success: true,
	}, nil
}
