package impl

import (
	"context"

	"gitlab.mashroom.com/mashroom/backend/common/api/process"
	"gitlab.mashroom.com/mashroom/backend/offer/model"
	"go.mongodb.org/mongo-driver/bson/primitive"
)

func (server *serverImpl) Ping(ctx context.Context, request *process.PingRequest) (*process.PingResponse, error) {
	return &process.PingResponse{Data: request.GetData()}, nil
}

func (server *serverImpl) Start(ctx context.Context, request *process.StartRequest) (*process.StartResponse, error) {
	err := server.setOfferWorkflowInstanceID(ctx, request)
	if err != nil {
		return nil, err
	}

	return &process.StartResponse{}, nil
}

func (server *serverImpl) Incident(
	ctx context.Context,
	request *process.IncidentRequest,
) (*process.IncidentResponse, error) {
	return &process.IncidentResponse{}, nil
}

func (server *serverImpl) Finish(
	ctx context.Context,
	request *process.FinishRequest,
) (*process.FinishResponse, error) {
	return &process.FinishResponse{}, nil
}

// setOfferWorkflowInstanceID sets workflow_instance_id to offer if is empty
func (server *serverImpl) setOfferWorkflowInstanceID(ctx context.Context, request *process.StartRequest) error {
	offerField, ok := request.GetPayload().GetFields()["offer"]
	if !ok {
		return nil
	}

	id, err := primitive.ObjectIDFromHex(offerField.GetStringValue())
	if err != nil {
		return err
	}

	offer, err := model.GetOfferByID(ctx, id)
	if err != nil {
		return err
	}

	if offer.WorkflowInstanceID != "" {
		return nil
	}

	offer.WorkflowInstanceID = request.GetProcess().GetInstance()

	err = model.UpdateOffer(ctx, offer)
	if err != nil {
		return err
	}

	return nil
}
