package rto

import (
	"context"
	"time"

	api "gitlab.mashroom.com/mashroom/backend/offer/api/rto"

	"go.mongodb.org/mongo-driver/bson/primitive"
)

type HistoryAction string

const (
	HistoryActionCreate HistoryAction = "create"
	HistoryActionUpdate HistoryAction = "update"
)

type History struct {
	CreatedAt       time.Time          `bson:"created_at"`
	UpdatedAt       time.Time          `bson:"updated_at"`
	PaidStatuses    *PaidStatuses      `bson:"paid_statuses"`
	Notifications   Notifications      `bson:"notifications"`
	Conditions      *Conditions        `bson:"conditions"`
	Landlord        *Landlord          `bson:"landlord"`
	PropertyIvr     string             `bson:"property_ivr"`
	PropertyAddress string             `bson:"property_address"`
	Property        string             `bson:"property"`
	PropertyListing string             `bson:"property_listing"`
	OfferID         string             `bson:"offer_id"`
	Action          HistoryAction      `bson:"action"`
	Tenants         []*Tenant          `bson:"tenants"`
	RentPlan        api.RentPlan       `bson:"rent_plan"`
	State           api.RtoState       `bson:"state"`
	ID              primitive.ObjectID `bson:"_id"`
}

type HistoryRepository interface {
	Create(ctx context.Context, history *History) error
	DeleteByRtoID(ctx context.Context, rtoID primitive.ObjectID) (int64, error)
}

func CreateHistory(action HistoryAction, now time.Time, rto *Rto) *History {
	return &History{
		ID:              primitive.NewObjectID(),
		Action:          action,
		CreatedAt:       now,
		OfferID:         rto.ID.Hex(),
		Conditions:      rto.Conditions,
		Property:        rto.Property,
		PropertyIvr:     rto.PropertyIvr,
		PropertyListing: rto.PropertyListing,
		PropertyAddress: rto.PropertyAddress,
		Landlord:        rto.Landlord,
		Tenants:         rto.Tenants,
		RentPlan:        rto.RentPlan,
		State:           rto.State,
		Notifications:   rto.Notifications,
		UpdatedAt:       rto.UpdatedAt,
		PaidStatuses:    rto.PaidStatuses,
	}
}
