package service

import (
	"fmt"

	"gitlab.mashroom.com/mashroom/backend/common/lib/constants"
	notification "gitlab.mashroom.com/mashroom/backend/notification/api"
	api "gitlab.mashroom.com/mashroom/backend/offer/api/rto"
)

type SendActiveRequest struct {
	LandlordID string
	PropertyID string
	RentPlan   api.RentPlan
}

func (sar *SendActiveRequest) Validate() error {
	if sar.LandlordID == "" {
		return fmt.Errorf(constants.ParameterCannotBeEmpty, "landlord_id")
	}
	if sar.PropertyID == "" {
		return fmt.Errorf(constants.ParameterCannotBeEmpty, "property_id")
	}
	if sar.RentPlan == api.RentPlan_RENT_PLAN_UNKNOWN {
		return fmt.Errorf("bad rent_plan value: %s", sar.RentPlan)
	}

	return nil
}

func (sar *SendActiveRequest) GetNotificationAction() notification.Action {
	switch sar.RentPlan {
	case api.RentPlan_RENT_PLAN_ESSENTIAL:
		return notification.Action_ACTION_RTO_STATE_ACTIVE_ESSENTIAL
	case api.RentPlan_RENT_PLAN_SECURED:
		return notification.Action_ACTION_RTO_STATE_ACTIVE_SECURED
	default:
		return notification.Action_ACTION_UNKNOWN
	}
}
