package usecase_test

import (
	"os"
	"testing"
	"time"

	"gitlab.mashroom.com/mashroom/backend/offer/api/rto"
	request "gitlab.mashroom.com/mashroom/backend/offer/internal/app/domain/rto/resource/request/usecase"

	. "github.com/smartystreets/goconvey/convey"
)

var (
	location    *time.Location
	locationErr error
)

func TestMain(m *testing.M) {
	mySetupFunction()
	retCode := m.Run()
	myTeardownFunction()
	os.Exit(retCode)
}

func mySetupFunction() {
	println("start rto request usecase package testing")

	location, locationErr = time.LoadLocation("Asia/Almaty")
	if locationErr != nil {
		panic(locationErr)
	}
}

func myTeardownFunction() {
	println("success end rto request package testing")
}

func TestCreateRequest_Validate(t *testing.T) {
	Convey("CreateRequest Validate", t, func() {
		req := &request.CreateRequest{}
		err := req.Validate()
		So(err, ShouldBeError)

		req.LandlordID = "test landlord id"
		err = req.Validate()
		So(err, ShouldBeError)

		req.PropertyID = "test property id"
		err = req.Validate()
		So(err, ShouldBeError)

		req.RentPlan = rto.RentPlan_RENT_PLAN_UNKNOWN
		err = req.Validate()
		So(err, ShouldBeError)

		req.RentPlan = rto.RentPlan_RENT_PLAN_ESSENTIAL
		err = req.Validate()
		So(err, ShouldBeNil)

		req.RentPlan = rto.RentPlan_RENT_PLAN_SECURED
		err = req.Validate()
		So(err, ShouldBeNil)
	})
}

func TestCreateTenancyRequest_Validate(t *testing.T) {
	Convey("CreateTenancyRequest Validate", t, func() {
		req := &request.CreateTenancyRequest{}
		err := req.Validate()
		So(err, ShouldBeError)

		req.ID = "test rto id"
		err = req.Validate()
		So(err, ShouldBeError)

		req.LandlordID = "test landlord id"
		err = req.Validate()
		So(err, ShouldBeError)

		req.PaymentData = &request.PaymentData{}
		req.PaymentData.RentAmount = 343526902400.666
		err = req.Validate()
		So(err, ShouldBeError)

		req.PaymentData.PaymentDay = 1
		err = req.Validate()
		So(err, ShouldBeError)

		req.PaymentData.StartDate = time.Date(1987, 7, 17, 6, 6, 6, 0, location)
		err = req.Validate()
		So(err, ShouldBeError)

		req.PaymentData.EndDate = time.Date(1999, 4, 21, 6, 6, 6, 0, location)
		err = req.Validate()
		So(err, ShouldBeError)

		req.LeadTenant = &request.User{
			FirstName: "aset-0-kulyash-0",
			LastName:  "dauren-0-daurennin-sheshesi-0",
			Email:     "aset_kulyash_dauren_daurennin_sheshesi.suki",
		}
		err = req.Validate()
		So(err, ShouldBeError)

		req.LeadTenant.Email = "aset_kulyash@dauren_daurennin_sheshesi.suki"
		err = req.Validate()
		So(err, ShouldBeNil)
	})
}
