package daemon_test

import (
	"testing"

	api "gitlab.mashroom.com/mashroom/backend/offer/api/rto"
	"gitlab.mashroom.com/mashroom/backend/offer/internal/app/service/rto/daemon"
	usecase "gitlab.mashroom.com/mashroom/backend/offer/internal/app/usecase/rto"
	"gitlab.mashroom.com/mashroom/backend/offer/test"

	. "github.com/smartystreets/goconvey/convey"
)

func TestFinishService(t *testing.T) {
	if test.CanRunTests() {
		Convey("FinishService", t, func() {
			updateStateToFinishedUseCase := usecase.NewUpdateStateToFinishedUseCase(repository,
				historyRepository)
			srv := daemon.NewFinishService(repository, updateStateToFinishedUseCase)
			So(srv, ShouldImplement, (*daemon.FinishService)(nil))

			Convey("Finish", func() {
				res, err := srv.Finish(ctx)
				So(err, ShouldBeNil)
				So(res.IDs, ShouldBeEmpty)

				rtoModel := test.CreateRto(ctx, t, repository, api.RtoState_RTO_STATE_PENDING_SETUP,
					test.KorshilerDogs, test.Chmolgani)

				res, err = srv.Finish(ctx)
				So(err, ShouldBeNil)
				So(res.IDs, ShouldBeEmpty)

				rtoModel.State = api.RtoState_RTO_STATE_ACTIVE
				err = repository.MustUpdate(ctx, rtoModel)
				So(err, ShouldBeNil)

				res, err = srv.Finish(ctx)
				So(err, ShouldBeNil)
				So(len(res.IDs), ShouldEqual, 1)
				So(res.IDs[0], ShouldEqual, rtoModel.ID.Hex())

				res, err = srv.Finish(ctx)
				So(err, ShouldBeNil)
				So(res.IDs, ShouldBeEmpty)

				test.DeleteRto(ctx, t, repository, historyRepository, rtoModel.ID, 1)

				rtoModelActive := test.CreateRto(ctx, t, repository, api.RtoState_RTO_STATE_ACTIVE,
					test.KorshilerDogs, test.Chmolgani)
				rtoModelAcitveRollingTenancy := test.CreateRto(ctx, t, repository,
					api.RtoState_RTO_STATE_ACTIVE_ROLLING_TENANCY, test.KorshilerDogs, test.Chmolgani)
				rtoModelEnding := test.CreateRto(ctx, t, repository,
					api.RtoState_RTO_STATE_ENDING, test.KorshilerDogs, test.Chmolgani)

				res, err = srv.Finish(ctx)
				So(err, ShouldBeNil)
				So(len(res.IDs), ShouldEqual, 3)

				res, err = srv.Finish(ctx)
				So(err, ShouldBeNil)
				So(res.IDs, ShouldBeEmpty)

				test.DeleteRto(ctx, t, repository, historyRepository, rtoModelActive.ID, 1)
				test.DeleteRto(ctx, t, repository, historyRepository, rtoModelAcitveRollingTenancy.ID, 1)
				test.DeleteRto(ctx, t, repository, historyRepository, rtoModelEnding.ID, 1)
			})
		})
	}
}
