package rto_test

import (
	"errors"
	"strings"
	"testing"

	account "gitlab.mashroom.com/mashroom/backend/account/api"
	mockaccount "gitlab.mashroom.com/mashroom/backend/account/api/mock"
	api "gitlab.mashroom.com/mashroom/backend/offer/api/rto"
	request "gitlab.mashroom.com/mashroom/backend/offer/internal/app/domain/rto/resource/request/service"
	"gitlab.mashroom.com/mashroom/backend/offer/internal/app/service/rto"
	"gitlab.mashroom.com/mashroom/backend/offer/test"

	"github.com/golang/mock/gomock"
	. "github.com/smartystreets/goconvey/convey"
)

func TestSecurityService(t *testing.T) {
	if test.CanRunTests() {
		Convey("SecurityService", t, func() {
			ctrl := gomock.NewController(t)
			defer ctrl.Finish()

			accountServiceClient := mockaccount.NewMockAccountServiceClient(ctrl)
			srv := rto.NewSecurityService(repository, accountServiceClient)
			So(srv, ShouldImplement, (*rto.SecurityService)(nil))

			Convey("IsVeriffValid", func() {
				req := &request.IsVeriffValidRequest{}
				err := srv.IsVeriffValid(ctx, req)
				So(err, ShouldBeError)
				So(strings.Contains(err.Error(), rto.IsVeriffValidNotValidRequestErrorText), ShouldBeTrue)

				testError := errors.New("mambeti - error")
				req.LandlordID = "kantar-oldi-mali"
				accountServiceClient.EXPECT().IsVeriffValid(ctx, &account.IsVeriffValidRequest{
					AccountId: req.LandlordID,
				}).Return(nil, testError)
				err = srv.IsVeriffValid(ctx, req)
				So(err, ShouldBeError)
				So(strings.Contains(err.Error(), rto.IsVeriffValidErrorText), ShouldBeTrue)

				accountServiceClient.EXPECT().IsVeriffValid(ctx, &account.IsVeriffValidRequest{
					AccountId: req.LandlordID,
				}).Return(&account.IsVeriffValidResponse{
					Success: false,
				}, nil)
				err = srv.IsVeriffValid(ctx, req)
				So(err, ShouldBeError)
				So(strings.Contains(err.Error(), rto.IsVeriffValidNotSuccessErrorText), ShouldBeTrue)

				accountServiceClient.EXPECT().IsVeriffValid(ctx, &account.IsVeriffValidRequest{
					AccountId: req.LandlordID,
				}).Return(&account.IsVeriffValidResponse{
					Success: true,
				}, nil)
				err = srv.IsVeriffValid(ctx, req)
				So(err, ShouldBeNil)
			})

			Convey("CanLandlordUpdateRto", func() {
				rtoModel := test.CreateRto(ctx, t, repository, api.RtoState_RTO_STATE_PENDING_SETUP,
					test.KorshilerDogs, test.Chmolgani)
				rtoID, landlordID := rtoModel.ID.Hex(), rtoModel.Landlord.User
				accountServiceClient.EXPECT().IsVeriffValid(ctx, &account.IsVeriffValidRequest{
					AccountId: landlordID,
				}).Return(&account.IsVeriffValidResponse{
					Success: true,
				}, nil)
				res, err := srv.CanLandlordUpdateRto(ctx, &request.CanLandlordUpdateRtoRequest{
					ID:              rtoID,
					LandlordID:      landlordID,
					NeedCheckVeriff: true,
				})
				So(err, ShouldBeNil)
				So(res, ShouldNotBeNil)

				test.DeleteRto(ctx, t, repository, historyRepository, rtoModel.ID, 0)
			})
		})
	}
}
