package rto_test

import (
	"strings"
	"testing"

	api "gitlab.mashroom.com/mashroom/backend/offer/api/rto"
	domain "gitlab.mashroom.com/mashroom/backend/offer/internal/app/domain/rto"
	request "gitlab.mashroom.com/mashroom/backend/offer/internal/app/domain/rto/resource/request/usecase"
	service "gitlab.mashroom.com/mashroom/backend/offer/internal/app/service/rto"
	"gitlab.mashroom.com/mashroom/backend/offer/internal/app/usecase/rto"
	"gitlab.mashroom.com/mashroom/backend/offer/test"

	. "github.com/smartystreets/goconvey/convey"
	"go.mongodb.org/mongo-driver/bson/primitive"
)

func TestAnswerFromTenantUseCase(t *testing.T) {
	if test.CanRunTests() {
		Convey("AnswerFromTenantUseCase", t, func() {
			securityService := service.NewSecurityService(repository, nil)
			useCase := rto.NewAnswerFromTenantUseCase(repository, historyRepository, securityService)

			Convey("AnswerFromTenant error "+rto.AnswerFromTenantNotValidStateErrorText, func() {
				id := primitive.NewObjectID()
				tenantUserID := "barligikimmagantuisiemescherti"
				rtoModel := &domain.Rto{
					ID:    id,
					State: api.RtoState_RTO_STATE_PENDING_SETUP,
					LeadTenant: &domain.Tenant{
						User: tenantUserID,
					},
				}
				err := repository.Create(ctx, rtoModel)
				So(err, ShouldBeNil)
				rtoID := id.Hex()

				req := &request.AnswerFromTenantRequest{
					ID:       rtoID,
					TenantID: tenantUserID,
					Accept:   false,
				}
				res, err := useCase.AnswerFromTenant(ctx, req)
				So(res, ShouldBeNil)
				So(err, ShouldBeError)
				So(strings.Contains(err.Error(), rto.AnswerFromTenantNotValidStateErrorText), ShouldBeTrue)

				rtoModel.State = api.RtoState_RTO_STATE_WAITING_TENANT_CONFIRMATION
				err = repository.MustUpdate(ctx, rtoModel)
				So(err, ShouldBeNil)

				Convey("AnswerFromTenant success", func() {
					res, err := useCase.AnswerFromTenant(ctx, req)
					So(err, ShouldBeNil)
					So(res, ShouldNotBeNil)
					So(res.Rto.State, ShouldEqual, api.RtoState_RTO_STATE_CANCELLED)

					test.DeleteRto(ctx, t, repository, historyRepository, rtoModel.ID, 1)
				})
			})
		})
	}
}
