package rto_test

import (
	"strings"
	"testing"

	"gitlab.mashroom.com/mashroom/backend/common/api/grpcctx"
	listing "gitlab.mashroom.com/mashroom/backend/listing/api"
	mocklisting "gitlab.mashroom.com/mashroom/backend/listing/api/mock"
	api "gitlab.mashroom.com/mashroom/backend/offer/api/rto"
	request "gitlab.mashroom.com/mashroom/backend/offer/internal/app/domain/rto/resource/request/usecase"
	"gitlab.mashroom.com/mashroom/backend/offer/internal/app/usecase/rto"
	"gitlab.mashroom.com/mashroom/backend/offer/test"

	"github.com/golang/mock/gomock"
	. "github.com/smartystreets/goconvey/convey"
)

func TestCreateByGuestUseCase(t *testing.T) {
	if test.CanRunTests() {
		Convey("CreateByGuestUseCase", t, func() {
			ctrl := gomock.NewController(t)
			defer ctrl.Finish()

			listingServiceClient := mocklisting.NewMockListingServiceClient(ctrl)
			useCase := rto.NewCreateByGuestUseCase(repository, historyRepository, listingServiceClient)

			propertyID := "test property id"
			listingServiceClient.EXPECT().VersionGet(grpcctx.WithServiceUser(ctx),
				gomock.Any()).Return(&listing.VersionGetResponse{
				Version: &listing.Version{PropertyId: propertyID},
			}, nil)
			req := &request.CreateByGuestRequest{
				RentPlan:   api.RentPlan_RENT_PLAN_ESSENTIAL,
				PropertyID: propertyID,
			}
			resp, err := useCase.CreateByGuest(ctx, req)
			So(err, ShouldBeNil)
			So(resp, ShouldNotBeNil)
			rtoModel := resp.Rto

			listingServiceClient.EXPECT().VersionGet(grpcctx.WithServiceUser(ctx),
				gomock.Any()).Return(&listing.VersionGetResponse{}, nil)
			resp, err = useCase.CreateByGuest(ctx, req)
			So(resp, ShouldBeNil)
			So(err, ShouldBeError)
			So(strings.Contains(err.Error(), rto.WithThisPropertyIDRtoAlreadyExistsErrorText), ShouldBeTrue)

			test.DeleteRto(ctx, t, repository, historyRepository, rtoModel.ID, 1)
		})
	}
}
