package rto_test

import (
	"testing"

	mockaccount "gitlab.mashroom.com/mashroom/backend/account/api/mock"
	extras "gitlab.mashroom.com/mashroom/backend/extras/api"
	mockinvoice "gitlab.mashroom.com/mashroom/backend/invoice/api/mock"
	api "gitlab.mashroom.com/mashroom/backend/offer/api/rto"
	requestservice "gitlab.mashroom.com/mashroom/backend/offer/internal/app/domain/rto/resource/request/service"
	request "gitlab.mashroom.com/mashroom/backend/offer/internal/app/domain/rto/resource/request/usecase"
	"gitlab.mashroom.com/mashroom/backend/offer/internal/app/domain/rto/resource/response"
	mockappservice "gitlab.mashroom.com/mashroom/backend/offer/internal/app/service/mock"
	mockservice "gitlab.mashroom.com/mashroom/backend/offer/internal/app/service/rto/mock"
	"gitlab.mashroom.com/mashroom/backend/offer/internal/app/usecase/rto"
	"gitlab.mashroom.com/mashroom/backend/offer/test"

	"github.com/golang/mock/gomock"
	. "github.com/smartystreets/goconvey/convey"
)

func TestCreateInvoiceUseCase(t *testing.T) {
	if test.CanRunTests() {
		Convey("CreateInvoiceUseCase", t, func() {
			ctrl := gomock.NewController(t)
			defer ctrl.Finish()

			accountServiceClient := mockaccount.NewMockAccountServiceClient(ctrl)
			invoiceServiceClient := mockinvoice.NewMockInvoiceServiceClient(ctrl)
			extrasService := mockappservice.NewMockExtrasService(ctrl)
			securityService := mockservice.NewMockSecurityService(ctrl)
			updateLeadTenantUserUseCase := rto.NewUpdateLeadTenantUserUseCase(
				repository, historyRepository, accountServiceClient)
			useCase := rto.NewCreateInvoiceUseCase(securityService, invoiceServiceClient,
				extrasService, updateLeadTenantUserUseCase)

			rtoModel := test.CreateRto(ctx, t, repository, api.RtoState_RTO_STATE_PENDING_SETUP,
				test.KorshilerDogs, test.Chmolgani)
			rtoID, landlordID := rtoModel.ID.Hex(), rtoModel.Landlord.User

			securityService.EXPECT().CanLandlordUpdateRto(ctx, &requestservice.CanLandlordUpdateRtoRequest{
				ID:              rtoID,
				LandlordID:      landlordID,
				NeedCheckVeriff: true,
			}).Return(&response.RtoResponse{Rto: rtoModel}, nil)
			extrasService.EXPECT().GetServicePriceSnapshot(ctx, extras.ProductTenantStandingOrder, test.Chmolgani).
				Return(&extras.ServiceSnapshot{}, nil)
			invoiceServiceClient.EXPECT().InvoiceCreate(ctx, gomock.Any()).Return(nil, nil)
			rtoCreateInvoiceResponse, err := useCase.CreateInvoice(ctx, &request.CreateInvoiceRequest{
				ID:         rtoID,
				LandlordID: landlordID,
			})
			So(err, ShouldBeNil)
			So(rtoCreateInvoiceResponse, ShouldNotBeNil)

			test.DeleteRto(ctx, t, repository, historyRepository, rtoModel.ID, 0)
		})
	}
}
