package rto_test

import (
	"testing"
	"time"

	account "gitlab.mashroom.com/mashroom/backend/account/api"
	mockaccount "gitlab.mashroom.com/mashroom/backend/account/api/mock"
	api "gitlab.mashroom.com/mashroom/backend/offer/api/rto"
	requestservice "gitlab.mashroom.com/mashroom/backend/offer/internal/app/domain/rto/resource/request/service"
	request "gitlab.mashroom.com/mashroom/backend/offer/internal/app/domain/rto/resource/request/usecase"
	"gitlab.mashroom.com/mashroom/backend/offer/internal/app/domain/rto/resource/response"
	mockservice "gitlab.mashroom.com/mashroom/backend/offer/internal/app/service/rto/mock"
	"gitlab.mashroom.com/mashroom/backend/offer/internal/app/usecase/rto"
	"gitlab.mashroom.com/mashroom/backend/offer/test"

	"github.com/golang/mock/gomock"
	. "github.com/smartystreets/goconvey/convey"
)

func TestCreateTenancyUseCase(t *testing.T) {
	if test.CanRunTests() {
		Convey("CreateTenancyUseCase", t, func() {
			ctrl := gomock.NewController(t)
			defer ctrl.Finish()

			accountServiceClient := mockaccount.NewMockAccountServiceClient(ctrl)
			securityService := mockservice.NewMockSecurityService(ctrl)
			useCase := rto.NewCreateTenancyUseCase(repository, historyRepository, securityService,
				accountServiceClient)

			rtoModel := test.CreateRto(ctx, t, repository, api.RtoState_RTO_STATE_PENDING_SETUP,
				test.ZhunusoviDogs, test.RentAmount)
			rtoID, landlordID := rtoModel.ID.Hex(), rtoModel.Landlord.User

			location, err := time.LoadLocation("Asia/Almaty")
			So(err, ShouldBeNil)

			securityService.EXPECT().CanLandlordUpdateRto(ctx, &requestservice.CanLandlordUpdateRtoRequest{
				ID:         rtoID,
				LandlordID: landlordID,
			}).Return(&response.RtoResponse{Rto: rtoModel}, nil)
			accountServiceClient.EXPECT().GetByEmail(ctx, &account.GetByEmailRequest{
				Email: test.ZhunusoviDogs,
			}).Return(&account.GetResponse{}, nil)
			createTenancyResponse, err := useCase.CreateTenancy(ctx,
				&request.CreateTenancyRequest{
					ID:         rtoID,
					LandlordID: landlordID,
					PaymentData: &request.PaymentData{
						StartDate:  time.Date(1987, 7, 17, 6, 6, 6, 0, location),
						EndDate:    time.Date(1999, 4, 21, 6, 6, 6, 0, location),
						RentAmount: test.RentAmount,
						PaymentDay: 17,
					},
					LeadTenant: &request.User{
						FirstName: "aset-0",
						LastName:  "kulyash-0",
						Email:     test.ZhunusoviDogs,
					},
				})
			So(err, ShouldBeNil)
			So(createTenancyResponse, ShouldNotBeNil)

			test.DeleteRto(ctx, t, repository, historyRepository, rtoModel.ID, 1)
		})
	}
}
