package rto_test

import (
	"strings"
	"testing"

	account "gitlab.mashroom.com/mashroom/backend/account/api"
	mockaccount "gitlab.mashroom.com/mashroom/backend/account/api/mock"
	"gitlab.mashroom.com/mashroom/backend/common/api/grpcctx"
	listing "gitlab.mashroom.com/mashroom/backend/listing/api"
	mocklisting "gitlab.mashroom.com/mashroom/backend/listing/api/mock"
	api "gitlab.mashroom.com/mashroom/backend/offer/api/rto"
	request "gitlab.mashroom.com/mashroom/backend/offer/internal/app/domain/rto/resource/request/usecase"
	"gitlab.mashroom.com/mashroom/backend/offer/internal/app/usecase/rto"
	"gitlab.mashroom.com/mashroom/backend/offer/test"

	"github.com/golang/mock/gomock"
	. "github.com/smartystreets/goconvey/convey"
)

func TestCreateUseCase(t *testing.T) {
	if test.CanRunTests() {
		Convey("CreateUseCase", t, func() {
			ctrl := gomock.NewController(t)
			defer ctrl.Finish()

			accountServiceClient := mockaccount.NewMockAccountServiceClient(ctrl)
			listingServiceClient := mocklisting.NewMockListingServiceClient(ctrl)
			useCase := rto.NewCreateUseCase(repository, historyRepository,
				accountServiceClient, listingServiceClient)

			landlordID := "test landlord id"
			propertyID := "test property id"
			listingServiceClient.EXPECT().VersionGet(grpcctx.WithServiceUser(ctx),
				gomock.Any()).Return(&listing.VersionGetResponse{
				Version: &listing.Version{PropertyId: propertyID},
			}, nil)
			accountServiceClient.EXPECT().Get(ctx, &account.GetRequest{
				Id: landlordID,
			}).Return(&account.GetResponse{
				Account: &account.Account{
					ViewedVideosIds: []int32{2065, 2032},
				},
			}, nil)
			req := &request.CreateRequest{
				LandlordID: landlordID,
				RentPlan:   api.RentPlan_RENT_PLAN_ESSENTIAL,
				PropertyID: propertyID,
			}
			resp, err := useCase.Create(ctx, req)
			So(err, ShouldBeNil)
			So(resp, ShouldNotBeNil)
			rtoModel := resp.Rto

			listingServiceClient.EXPECT().VersionGet(grpcctx.WithServiceUser(ctx),
				gomock.Any()).Return(&listing.VersionGetResponse{}, nil)
			resp, err = useCase.Create(ctx, req)
			So(resp, ShouldBeNil)
			So(err, ShouldBeError)
			So(strings.Contains(err.Error(), rto.WithThisPropertyIDRtoAlreadyExistsErrorText), ShouldBeTrue)

			test.DeleteRto(ctx, t, repository, historyRepository, rtoModel.ID, 1)
		})
	}
}
