package rto

import (
	"context"

	domain "gitlab.mashroom.com/mashroom/backend/offer/internal/app/domain/rto"
	requestservice "gitlab.mashroom.com/mashroom/backend/offer/internal/app/domain/rto/resource/request/service"
	request "gitlab.mashroom.com/mashroom/backend/offer/internal/app/domain/rto/resource/request/usecase"
	"gitlab.mashroom.com/mashroom/backend/offer/internal/app/domain/rto/resource/response"
	service "gitlab.mashroom.com/mashroom/backend/offer/internal/app/service/rto"

	"github.com/pkg/errors"
	"go.mongodb.org/mongo-driver/mongo"
)

type GetByPropertyIDUseCase interface {
	GetByPropertyID(ctx context.Context, req *request.GetByPropertyIDRequest) (*response.RtoResponse, error)
}

type getByPropertyIDUseCase struct {
	repository      domain.Repository
	securityService service.SecurityService
}

func NewGetByPropertyIDUseCase(repository domain.Repository,
	securityService service.SecurityService) GetByPropertyIDUseCase {
	return &getByPropertyIDUseCase{
		securityService: securityService,
		repository:      repository,
	}
}

func (gbpiuc *getByPropertyIDUseCase) GetByPropertyID(ctx context.Context, req *request.GetByPropertyIDRequest) (
	*response.RtoResponse, error) {
	err := req.Validate()
	if err != nil {
		return nil, errors.Wrap(err, "rto getByPropertyIDUseCase req.Validate")
	}

	rto, err := gbpiuc.repository.FindByPropertyID(ctx, req.PropertyID)
	if err != nil {
		if errors.Is(err, mongo.ErrNoDocuments) {
			return nil, nil
		}

		return nil, errors.Wrap(err, "rto getByPropertyIDUseCase repository.FindByPropertyID")
	}

	err = gbpiuc.securityService.CanUserReadRto(ctx, &requestservice.CanUserReadRtoRequest{
		Rto:    rto,
		UserID: req.UserID,
	})
	if err != nil {
		return nil, errors.Wrap(err, "rto getByPropertyIDUseCase securityService.CanReadAccess")
	}

	return &response.RtoResponse{
		Rto: rto,
	}, nil
}
