package rto_test

import (
	"strings"
	"testing"

	api "gitlab.mashroom.com/mashroom/backend/offer/api/rto"
	domain "gitlab.mashroom.com/mashroom/backend/offer/internal/app/domain/rto"
	request "gitlab.mashroom.com/mashroom/backend/offer/internal/app/domain/rto/resource/request/usecase"
	service "gitlab.mashroom.com/mashroom/backend/offer/internal/app/service/rto"
	"gitlab.mashroom.com/mashroom/backend/offer/internal/app/usecase/rto"
	"gitlab.mashroom.com/mashroom/backend/offer/test"

	. "github.com/smartystreets/goconvey/convey"
	"go.mongodb.org/mongo-driver/bson/primitive"
)

func TestGetByPropertyIDUseCase(t *testing.T) {
	if test.CanRunTests() {
		Convey("GetByPropertyIDUseCase", t, func() {
			securityService := service.NewSecurityService(repository, nil)
			useCase := rto.NewGetByPropertyIDUseCase(repository, securityService)

			Convey("GetByPropertyIDUseCase empty", func() {
				propertyID, userID := "chertivsektomnetuisiemes", "chmorikivsektomnetuisiemes"
				req := &request.GetByPropertyIDRequest{
					PropertyID: propertyID,
					UserID:     userID,
				}
				resp, err := useCase.GetByPropertyID(ctx, req)
				So(err, ShouldBeNil)
				So(resp, ShouldBeNil)

				Convey("GetByPropertyIDUseCase success", func() {
					id := primitive.NewObjectID()
					rtoModel := &domain.Rto{
						ID:       id,
						Property: propertyID,
						Landlord: &domain.Landlord{
							User: userID,
						},
						State: api.RtoState_RTO_STATE_PENDING_SETUP,
					}
					err = repository.Create(ctx, rtoModel)
					So(err, ShouldBeNil)

					resp, err = useCase.GetByPropertyID(ctx, req)
					So(err, ShouldBeNil)
					So(resp.Rto.ID, ShouldEqual, id)

					Convey("GetByPropertyIDUseCase error"+service.CanUserReadRtoNotValidUserErrorText, func() {
						req.UserID = "barligikimmagantuisiemesolsin"
						resp, err = useCase.GetByPropertyID(ctx, req)
						So(err, ShouldNotBeNil)
						So(resp, ShouldBeNil)
						So(strings.Contains(err.Error(), service.CanUserReadRtoNotValidUserErrorText), ShouldBeTrue)

						Convey("GetByPropertyIDUseCase error"+service.CanUserReadRtoNotValidStateErrorText, func() {
							tenantUserID := "barligikimmagantuisiemescherti"
							rtoModel.Landlord.User = ""
							rtoModel.LeadTenant = &domain.Tenant{
								User: tenantUserID,
							}
							So(repository.MustUpdate(ctx, rtoModel), ShouldBeNil)

							req.UserID = tenantUserID
							resp, err = useCase.GetByPropertyID(ctx, req)
							So(err, ShouldNotBeNil)
							So(resp, ShouldBeNil)
							So(strings.Contains(err.Error(), service.CanUserReadRtoNotValidStateErrorText),
								ShouldBeTrue)

							deletedCount, err := repository.DeleteByID(ctx, id)
							So(err, ShouldBeNil)
							So(deletedCount, ShouldEqual, 1)
						})
					})
				})
			})
		})
	}
}
