package rto_test

import (
	"testing"

	api "gitlab.mashroom.com/mashroom/backend/offer/api/rto"
	domain "gitlab.mashroom.com/mashroom/backend/offer/internal/app/domain/rto"
	request "gitlab.mashroom.com/mashroom/backend/offer/internal/app/domain/rto/resource/request/usecase"
	service "gitlab.mashroom.com/mashroom/backend/offer/internal/app/service/rto"
	"gitlab.mashroom.com/mashroom/backend/offer/internal/app/usecase/rto"
	"gitlab.mashroom.com/mashroom/backend/offer/test"

	. "github.com/smartystreets/goconvey/convey"
	"go.mongodb.org/mongo-driver/bson/primitive"
)

func TestGetByPropertyIDsUseCase(t *testing.T) {
	if test.CanRunTests() {
		Convey("GetByPropertyIDsUseCase", t, func() {
			securityService := service.NewSecurityService(repository, nil)
			useCase := rto.NewGetByPropertyIDsUseCase(repository, securityService)

			Convey("GetByPropertyIDsUseCase empty", func() {
				propertyID, userID := "chertivsektomnetuisiemes", "chmorikivsektomnetuisiemes"
				req := &request.GetByPropertyIDsRequest{
					PropertyIDs: []string{propertyID},
					UserID:      userID,
				}
				resp, err := useCase.GetByPropertyIDs(ctx, req)
				So(err, ShouldBeNil)
				So(len(resp.Rtos), ShouldBeZeroValue)

				Convey("GetByPropertyIDsUseCase success", func() {
					id := primitive.NewObjectID()
					rtoModel := &domain.Rto{
						ID:       id,
						Property: propertyID,
						Landlord: &domain.Landlord{
							User: userID,
						},
						State: api.RtoState_RTO_STATE_PENDING_SETUP,
					}
					err = repository.Create(ctx, rtoModel)
					So(err, ShouldBeNil)

					resp, err = useCase.GetByPropertyIDs(ctx, req)
					So(err, ShouldBeNil)
					So(resp.Rtos[0].ID, ShouldEqual, id)
				})
			})
		})
	}
}
