package rto_test

import (
	"strings"
	"testing"
	"time"

	api "gitlab.mashroom.com/mashroom/backend/offer/api/rto"
	requestservice "gitlab.mashroom.com/mashroom/backend/offer/internal/app/domain/rto/resource/request/service"
	request "gitlab.mashroom.com/mashroom/backend/offer/internal/app/domain/rto/resource/request/usecase"
	"gitlab.mashroom.com/mashroom/backend/offer/internal/app/domain/rto/resource/response"
	mockservice "gitlab.mashroom.com/mashroom/backend/offer/internal/app/service/rto/mock"
	"gitlab.mashroom.com/mashroom/backend/offer/internal/app/usecase/rto"
	"gitlab.mashroom.com/mashroom/backend/offer/test"

	"github.com/golang/mock/gomock"
	. "github.com/smartystreets/goconvey/convey"
)

func TestUpdateEndDateUseCase(t *testing.T) {
	if test.CanRunTests() {
		Convey("UpdateDateUseCase", t, func() {
			ctrl := gomock.NewController(t)
			defer ctrl.Finish()

			securityService := mockservice.NewMockSecurityService(ctrl)
			useCase := rto.NewUpdateEndDateUseCase(repository, historyRepository, securityService)

			Convey("UpdateEndDate error "+rto.UpdateEndDateNotValidStateErrorText, func() {
				rtoModel := test.CreateRto(ctx, t, repository, api.RtoState_RTO_STATE_PENDING_SETUP,
					test.KorshilerDogs, test.Chmolgani)
				rtoID, landlordID := rtoModel.ID.Hex(), rtoModel.Landlord.User

				securityService.EXPECT().CanLandlordUpdateRto(ctx, &requestservice.CanLandlordUpdateRtoRequest{
					ID:              rtoID,
					LandlordID:      landlordID,
					NeedCheckVeriff: true,
				}).Return(&response.RtoResponse{Rto: rtoModel}, nil)
				rtoUpdateEndDateRequest := &request.UpdateEndDateRequest{
					ID:         rtoID,
					LandlordID: landlordID,
					EndDate:    rtoModel.CreatedAt.Add(-time.Hour),
				}
				res, err := useCase.UpdateEndDate(ctx, rtoUpdateEndDateRequest)
				So(res, ShouldBeNil)
				So(err, ShouldBeError)
				So(strings.Contains(err.Error(), rto.UpdateEndDateNotValidStateErrorText), ShouldBeTrue)

				rtoModel.State = api.RtoState_RTO_STATE_ENDING
				err = repository.MustUpdate(ctx, rtoModel)
				So(err, ShouldBeNil)

				Convey("UpdateEndDate error "+rto.UpdateEndDateNotValidEndDateErrorText, func() {
					securityService.EXPECT().CanLandlordUpdateRto(ctx, &requestservice.CanLandlordUpdateRtoRequest{
						ID:              rtoID,
						LandlordID:      landlordID,
						NeedCheckVeriff: true,
					}).Return(&response.RtoResponse{Rto: rtoModel}, nil)
					res, err = useCase.UpdateEndDate(ctx, rtoUpdateEndDateRequest)
					So(res, ShouldBeNil)
					So(err, ShouldBeError)
					So(strings.Contains(err.Error(), rto.UpdateEndDateNotValidEndDateErrorText),
						ShouldBeTrue)

					Convey("UpdateEndDate success", func() {
						securityService.EXPECT().CanLandlordUpdateRto(ctx, &requestservice.CanLandlordUpdateRtoRequest{
							ID:              rtoID,
							LandlordID:      landlordID,
							NeedCheckVeriff: true,
						}).Return(&response.RtoResponse{Rto: rtoModel}, nil)
						rtoUpdateEndDateRequest.EndDate = rtoModel.CreatedAt.Add(2 * time.Hour)
						res, err = useCase.UpdateEndDate(ctx, rtoUpdateEndDateRequest)
						So(err, ShouldBeNil)
						So(res, ShouldNotBeNil)

						test.DeleteRto(ctx, t, repository, historyRepository, rtoModel.ID, 1)
					})
				})
			})
		})
	}
}
