package rto_test

import (
	"testing"

	api "gitlab.mashroom.com/mashroom/backend/offer/api/rto"
	requestservice "gitlab.mashroom.com/mashroom/backend/offer/internal/app/domain/rto/resource/request/service"
	request "gitlab.mashroom.com/mashroom/backend/offer/internal/app/domain/rto/resource/request/usecase"
	"gitlab.mashroom.com/mashroom/backend/offer/internal/app/domain/rto/resource/response"
	mockservice "gitlab.mashroom.com/mashroom/backend/offer/internal/app/service/rto/mock"
	"gitlab.mashroom.com/mashroom/backend/offer/internal/app/usecase/rto"
	"gitlab.mashroom.com/mashroom/backend/offer/test"

	"github.com/golang/mock/gomock"
	. "github.com/smartystreets/goconvey/convey"
)

func TestUpdateStateUseCase(t *testing.T) {
	if test.CanRunTests() {
		Convey("UpdateStateUseCase", t, func() {
			ctrl := gomock.NewController(t)
			defer ctrl.Finish()

			securityService := mockservice.NewMockSecurityService(ctrl)
			useCase := rto.NewUpdateStateUseCase(repository, historyRepository, securityService)

			rtoModel := test.CreateRto(ctx, t, repository, api.RtoState_RTO_STATE_PENDING_SETUP,
				test.KorshilerDogs, test.Chmolgani)
			rtoID, landlordID := rtoModel.ID.Hex(), rtoModel.Landlord.User

			securityService.EXPECT().CanLandlordUpdateRto(ctx, &requestservice.CanLandlordUpdateRtoRequest{
				ID:              rtoID,
				LandlordID:      landlordID,
				NeedCheckVeriff: true,
			}).Return(&response.RtoResponse{Rto: rtoModel}, nil)
			res, err := useCase.UpdateState(ctx, &request.UpdateStateRequest{
				ID:         rtoID,
				LandlordID: landlordID,
				State:      api.RtoState_RTO_STATE_WAITING_FOR_STANDING_ORDER_SETUP,
			})
			So(err, ShouldBeNil)
			So(res, ShouldNotBeNil)

			test.DeleteRto(ctx, t, repository, historyRepository, rtoModel.ID, 1)
		})
	}
}
