package rto_test

import (
	"strings"
	"testing"

	api "gitlab.mashroom.com/mashroom/backend/offer/api/rto"
	request "gitlab.mashroom.com/mashroom/backend/offer/internal/app/domain/rto/resource/request/usecase"
	"gitlab.mashroom.com/mashroom/backend/offer/internal/app/usecase/rto"
	"gitlab.mashroom.com/mashroom/backend/offer/test"

	. "github.com/smartystreets/goconvey/convey"
)

func TestUpdateStateToActiveRollingTenancyUseCase(t *testing.T) {
	if test.CanRunTests() {
		Convey("UpdateStateToActiveRollingTenancyUseCase", t, func() {
			useCase := rto.NewUpdateStateToActiveRollingTenancyUseCase(repository, historyRepository)

			Convey("UpdateStateToActiveRollingTenancy error "+
				rto.UpdateStateToActiveRollingTenancyNotValidStateErrorText, func() {
				rtoModel := test.CreateRto(ctx, t, repository, api.RtoState_RTO_STATE_PENDING_SETUP,
					test.KorshilerDogs, test.Chmolgani)
				rtoID := rtoModel.ID.Hex()
				req := &request.UpdateStateToActiveRollingTenancyRequest{
					ID: rtoID,
				}
				res, err := useCase.UpdateStateToActiveRollingTenancy(ctx, req)
				So(res, ShouldBeNil)
				So(err, ShouldBeError)
				So(strings.Contains(err.Error(), rto.UpdateStateToActiveRollingTenancyNotValidStateErrorText),
					ShouldBeTrue)

				rtoModel.State = api.RtoState_RTO_STATE_ACTIVE
				err = repository.MustUpdate(ctx, rtoModel)
				So(err, ShouldBeNil)

				Convey("UpdateStateToActiveRollingTenancy success", func() {
					res, err := useCase.UpdateStateToActiveRollingTenancy(ctx, req)
					So(err, ShouldBeNil)
					So(res, ShouldNotBeNil)

					test.DeleteRto(ctx, t, repository, historyRepository, rtoModel.ID, 1)
				})
			})
		})
	}
}
