package rto_test

import (
	"strings"
	"testing"

	api "gitlab.mashroom.com/mashroom/backend/offer/api/rto"
	request "gitlab.mashroom.com/mashroom/backend/offer/internal/app/domain/rto/resource/request/usecase"
	mockservicerto "gitlab.mashroom.com/mashroom/backend/offer/internal/app/service/rto/mock"
	"gitlab.mashroom.com/mashroom/backend/offer/internal/app/usecase/rto"
	"gitlab.mashroom.com/mashroom/backend/offer/test"

	"github.com/golang/mock/gomock"
	. "github.com/smartystreets/goconvey/convey"
)

func TestUpdateStateToActiveUseCase(t *testing.T) {
	if test.CanRunTests() {
		Convey("UpdateStateToActiveUseCase", t, func() {
			ctrl := gomock.NewController(t)
			defer ctrl.Finish()

			rtoNotificationService := mockservicerto.NewMockNotificationService(ctrl)
			useCase := rto.NewUpdateStateToActiveUseCase(repository, historyRepository, rtoNotificationService)

			Convey("UpdateStateToActive error "+
				rto.UpdateStateToActiveNotValidStateErrorText, func() {
				rtoModel := test.CreateRto(ctx, t, repository, api.RtoState_RTO_STATE_PENDING_SETUP,
					test.KorshilerDogs, test.Chmolgani)
				rtoID := rtoModel.ID.Hex()
				req := &request.UpdateStateToActiveRequest{
					ID: rtoID,
				}
				res, err := useCase.UpdateStateToActive(ctx, req)
				So(res, ShouldBeNil)
				So(err, ShouldBeError)
				So(strings.Contains(err.Error(), rto.UpdateStateToActiveNotValidStateErrorText),
					ShouldBeTrue)

				rtoModel.State = api.RtoState_RTO_STATE_WAITING_FOR_FIRST_RENT_PAYMENT
				err = repository.MustUpdate(ctx, rtoModel)
				So(err, ShouldBeNil)

				Convey("UpdateStateToActive success", func() {
					rtoNotificationService.EXPECT().SendActive(ctx, gomock.Any()).Return(nil)
					res, err := useCase.UpdateStateToActive(ctx, req)
					So(err, ShouldBeNil)
					So(res, ShouldNotBeNil)

					test.DeleteRto(ctx, t, repository, historyRepository, rtoModel.ID, 1)
				})
			})
		})
	}
}
