package rto_test

import (
	"strings"
	"testing"
	"time"

	api "gitlab.mashroom.com/mashroom/backend/offer/api/rto"
	request "gitlab.mashroom.com/mashroom/backend/offer/internal/app/domain/rto/resource/request/usecase"
	"gitlab.mashroom.com/mashroom/backend/offer/internal/app/usecase/rto"
	"gitlab.mashroom.com/mashroom/backend/offer/test"

	. "github.com/smartystreets/goconvey/convey"
)

func TestUpdateStateToCancelledUseCase(t *testing.T) {
	if test.CanRunTests() {
		Convey("UpdateStateToCancelledUseCase", t, func() {
			useCase := rto.NewUpdateStateToCancelledUseCase(repository, historyRepository)

			Convey("UpdateStateToCancelled error "+
				rto.UpdateStateToCancelledNotValidStateErrorText, func() {
				rtoModel := test.CreateRto(ctx, t, repository, api.RtoState_RTO_STATE_PENDING_SETUP,
					test.KorshilerDogs, test.Chmolgani)
				rtoID := rtoModel.ID.Hex()
				req := &request.UpdateStateToCancelledRequest{
					ID: rtoID,
				}
				res, err := useCase.UpdateStateToCancelled(ctx, req)
				So(res, ShouldBeNil)
				So(err, ShouldBeError)
				So(strings.Contains(err.Error(), rto.UpdateStateToCancelledNotValidStateErrorText), ShouldBeTrue)

				Convey("UpdateStateToCancelled error "+
					rto.UpdateStateToCancelledNotValidRtoEndDateErrorText, func() {
					rtoModel.State = api.RtoState_RTO_STATE_ENDING
					rtoModel.Conditions.EndDate = time.Now().Add(59.5 * 24 * time.Hour)
					err = repository.MustUpdate(ctx, rtoModel)
					So(err, ShouldBeNil)

					res, err = useCase.UpdateStateToCancelled(ctx, req)
					So(res, ShouldBeNil)
					So(err, ShouldBeError)
					So(strings.Contains(err.Error(),
						rto.UpdateStateToCancelledNotValidRtoEndDateErrorText), ShouldBeTrue)

					rtoModel.Conditions.EndDate = time.Now().Add(60.5 * 24 * time.Hour)
					err = repository.MustUpdate(ctx, rtoModel)
					So(err, ShouldBeNil)

					Convey("UpdateStateToCancelled success", func() {
						res, err := useCase.UpdateStateToCancelled(ctx, req)
						So(err, ShouldBeNil)
						So(res, ShouldNotBeNil)

						test.DeleteRto(ctx, t, repository, historyRepository, rtoModel.ID, 1)
					})
				})
			})
		})
	}
}
