package rto_test

import (
	"strings"
	"testing"

	api "gitlab.mashroom.com/mashroom/backend/offer/api/rto"
	request "gitlab.mashroom.com/mashroom/backend/offer/internal/app/domain/rto/resource/request/usecase"
	mockservicerto "gitlab.mashroom.com/mashroom/backend/offer/internal/app/service/rto/mock"
	"gitlab.mashroom.com/mashroom/backend/offer/internal/app/usecase/rto"
	"gitlab.mashroom.com/mashroom/backend/offer/test"

	"github.com/golang/mock/gomock"
	. "github.com/smartystreets/goconvey/convey"
)

func TestUpdateStateToWaitingForFirstRentPaymentUseCase(t *testing.T) {
	if test.CanRunTests() {
		Convey("UpdateStateToWaitingForFirstRentPaymentUseCase", t, func() {
			ctrl := gomock.NewController(t)
			defer ctrl.Finish()

			rtoNotificationService := mockservicerto.NewMockNotificationService(ctrl)
			useCase := rto.NewUpdateStateToWaitingForFirstRentPaymentUseCase(repository,
				historyRepository, rtoNotificationService)

			Convey("UpdateStateToWaitingForFirstRentPayment error "+
				rto.UpdateStateToWaitingForFirstRentPaymentNotValidStateErrorText, func() {
				rtoModel := test.CreateRto(ctx, t, repository, api.RtoState_RTO_STATE_PENDING_SETUP,
					test.KorshilerDogs, test.Chmolgani)
				rtoID := rtoModel.ID.Hex()
				req := &request.UpdateStateToWaitingForFirstRentPaymentRequest{
					ID: rtoID,
				}
				res, err := useCase.UpdateStateToWaitingForFirstRentPayment(ctx, req)
				So(res, ShouldBeNil)
				So(err, ShouldBeError)
				So(strings.Contains(err.Error(), rto.UpdateStateToWaitingForFirstRentPaymentNotValidStateErrorText),
					ShouldBeTrue)

				rtoModel.State = api.RtoState_RTO_STATE_WAITING_FOR_STANDING_ORDER_SETUP
				err = repository.MustUpdate(ctx, rtoModel)
				So(err, ShouldBeNil)

				Convey("UpdateStateToWaitingForFirstRentPayment success", func() {
					rtoNotificationService.EXPECT().SendWaitingForFirstRentPayment(ctx, gomock.Any()).Return(nil)
					res, err := useCase.UpdateStateToWaitingForFirstRentPayment(ctx, req)
					So(err, ShouldBeNil)
					So(res, ShouldNotBeNil)

					test.DeleteRto(ctx, t, repository, historyRepository, rtoModel.ID, 1)
				})
			})
		})
	}
}
