package rto_test

import (
	"strings"
	"testing"

	mockaccount "gitlab.mashroom.com/mashroom/backend/account/api/mock"
	api "gitlab.mashroom.com/mashroom/backend/offer/api/rto"
	request "gitlab.mashroom.com/mashroom/backend/offer/internal/app/domain/rto/resource/request/usecase"
	mockservicerto "gitlab.mashroom.com/mashroom/backend/offer/internal/app/service/rto/mock"
	"gitlab.mashroom.com/mashroom/backend/offer/internal/app/usecase/rto"
	"gitlab.mashroom.com/mashroom/backend/offer/test"

	"github.com/golang/mock/gomock"
	. "github.com/smartystreets/goconvey/convey"
)

func TestUpdateStateToWaitingForStandingOrderSetupUseCase(t *testing.T) {
	if test.CanRunTests() {
		Convey("UpdateStateToWaitingForStandingOrderSetupUseCase", t, func() {
			ctrl := gomock.NewController(t)
			defer ctrl.Finish()

			accountServiceClient := mockaccount.NewMockAccountServiceClient(ctrl)
			rtoNotificationService := mockservicerto.NewMockNotificationService(ctrl)
			updateLeadTenantUserUseCase := rto.NewUpdateLeadTenantUserUseCase(
				repository, historyRepository, accountServiceClient)
			useCase := rto.NewUpdateStateToWaitingForStandingOrderSetupUseCase(repository,
				historyRepository, updateLeadTenantUserUseCase, rtoNotificationService)

			Convey("UpdateStateToWaitingForStandingOrderSetup error "+
				rto.UpdateStateToWaitingForStandingOrderSetupNotValidStateErrorText, func() {
				rtoModel := test.CreateRto(ctx, t, repository,
					api.RtoState_RTO_STATE_WAITING_FOR_STANDING_ORDER_SETUP,
					test.KorshilerDogs, test.Chmolgani)
				rtoID, landlordID := rtoModel.ID.Hex(), rtoModel.Landlord.User
				req := &request.UpdateStateToWaitingForStandingOrderSetupRequest{
					ID:         rtoID,
					LandlordID: landlordID,
				}
				res, err := useCase.UpdateStateToWaitingForStandingOrderSetup(ctx, req)
				So(res, ShouldBeNil)
				So(err, ShouldBeError)
				So(strings.Contains(err.Error(), rto.UpdateStateToWaitingForStandingOrderSetupNotValidStateErrorText),
					ShouldBeTrue)

				rtoModel.State = api.RtoState_RTO_STATE_PENDING_SETUP
				err = repository.MustUpdate(ctx, rtoModel)
				So(err, ShouldBeNil)

				Convey("UpdateStateToWaitingForStandingOrderSetup success", func() {
					rtoNotificationService.EXPECT().SendWaitingForStandingOrderSetup(ctx, gomock.Any()).Return(nil)
					res, err := useCase.UpdateStateToWaitingForStandingOrderSetup(ctx, req)
					So(err, ShouldBeNil)
					So(res, ShouldNotBeNil)

					test.DeleteRto(ctx, t, repository, historyRepository, rtoModel.ID, 1)
				})
			})
		})
	}
}
