package connections

import (
	account "gitlab.mashroom.com/mashroom/backend/account/api"
	closecom "gitlab.mashroom.com/mashroom/backend/closecom/api"
	"gitlab.mashroom.com/mashroom/backend/common/generic/env"
	"gitlab.mashroom.com/mashroom/backend/common/generic/grpc"
	contract "gitlab.mashroom.com/mashroom/backend/contract/api"
	document "gitlab.mashroom.com/mashroom/backend/document/api"
	expenses "gitlab.mashroom.com/mashroom/backend/expenses/api"
	extras "gitlab.mashroom.com/mashroom/backend/extras/api"
	identity "gitlab.mashroom.com/mashroom/backend/identity/api"
	invitation "gitlab.mashroom.com/mashroom/backend/invitation/api"
	invoice "gitlab.mashroom.com/mashroom/backend/invoice/api"
	listing "gitlab.mashroom.com/mashroom/backend/listing/api"
	mixpanel "gitlab.mashroom.com/mashroom/backend/mixpanel/api"
	notification "gitlab.mashroom.com/mashroom/backend/notification/api"
	property "gitlab.mashroom.com/mashroom/backend/property/api"
	schedule "gitlab.mashroom.com/mashroom/backend/schedule/api"
	signature "gitlab.mashroom.com/mashroom/backend/signature/api"
	tenancy "gitlab.mashroom.com/mashroom/backend/tenancy/api"
	viewing "gitlab.mashroom.com/mashroom/backend/viewing/api"
	zeebe "gitlab.mashroom.com/mashroom/backend/zeebe/api"
)

// SelfRemote is Offer service network address.
var SelfRemote = env.ResolveString("SELF_REMOTE", "mashroom-offer:10000")

// Instance of Mashroom service client.
var (
	ZeebeExporter = zeebe.NewZeebeProcessClient(grpc.Dial(
		env.ResolveString("BPMS_EXPORTER_URL", "mashroom-zeebe:10000"),
		"ZeebeExporter",
	))
	InvitationService = invitation.NewInvitationServiceClient(grpc.Dial(
		env.ResolveString("INVITATION_SERVICE_URL", "mashroom-invitation:10000"),
		"Invitation",
	))
	SignatureService = signature.NewSignatureServiceClient(grpc.Dial(
		env.ResolveString("SIGNATURE_SERVICE_URL", "mashroom-signature:10000"),
		"Signature",
	))
	ExtrasService = extras.NewExtrasServiceClient(grpc.Dial(
		env.ResolveString("EXTRAS_SERVICE_URL", "mashroom-extras:10000"),
		"Extras",
	))
	PropertyService = property.NewPropertyServiceClient(grpc.Dial(
		env.ResolveString("PROPERTY_SERVICE_URL", "mashroom-property:10000"),
		"Property",
	))
	TenancyService = tenancy.NewTenancyServiceClient(grpc.Dial(
		env.ResolveString("TENANCY_SERVICE_URL", "mashroom-tenancy:10000"),
		"Tenancy",
	))
	ListingService = listing.NewListingServiceClient(grpc.Dial(
		env.ResolveString("LISTING_SERVICE_URL", "mashroom-listing:10000"),
		"Listing",
	))
	IdentityService = identity.NewIdentityServiceClient(grpc.Dial(
		env.ResolveString("IDENTITY_SERVICE_URL", "mashroom-identity:10000"),
		"Identity",
	))
	DocumentService = document.NewDocumentServiceClient(grpc.Dial(
		env.ResolveString("DOCUMENT_SERVICE_URL", "mashroom-document:10000"),
		"Document",
	))
	ContractService = contract.NewContractServiceClient(grpc.Dial(
		env.ResolveString("CONTRACT_SERVICE_URL", "mashroom-contract:10000"),
		"Contract",
	))
	AccountService = account.NewAccountServiceClient(grpc.Dial(
		env.ResolveString("ACCOUNT_SERVICE_URL", "mashroom-account:10000"),
		"Account",
	))
	NotificationService = notification.NewNotificationServiceClient(grpc.Dial(
		env.ResolveString("NOTIFICATION_SERVICE_URL", "mashroom-notification:10000"),
		"Notification",
	))
	InvoiceService = invoice.NewInvoiceServiceClient(grpc.Dial(
		env.ResolveString("INVOICE_SERVICE_URL", "mashroom-invoice:10000"),
		"Invoice",
	))
	ScheduleService = schedule.NewScheduleServiceClient(grpc.Dial(
		env.ResolveString("SCHEDULE_SERVICE_URL", "mashroom-schedule:10000"),
		"Schedule",
	))
	MixpanelService = mixpanel.NewMixpanelServiceClient(grpc.Dial(
		env.ResolveString("MIXPANEL_SERVICE_URL", "mashroom-mixpanel:10000"),
		"Mixpanel",
	))
	ViewingService = viewing.NewViewingServiceClient(grpc.Dial(
		env.ResolveString("VIEWING_SERVICE_URL", "mashroom-viewing:10000"),
		"Viewing",
	))
	ClosecomService = closecom.NewClosecomServiceClient(grpc.Dial(
		env.ResolveString("CLOSECOM_SERVICE_URL", "mashroom-closecom:10000"),
		"Closecom",
	))
	ExpensesService = expenses.NewExpensesServiceClient(grpc.Dial(
		env.ResolveString("EXPENSES_SERVICE_URL", "mashroom-expenses:10000"),
		"Expenses",
	))
	ExpensesInvoiceNotificationService = invoice.NewInvoiceNotifyServiceClient(grpc.Dial(
		env.ResolveString("EXPENSES_SERVICE_URL", "mashroom-expenses:10000"),
		"Expenses",
	))
)
