// Package hourly contains periodic task handlers that are executed every hour
package hourly

import (
	"context"

	"gitlab.mashroom.com/mashroom/backend/common/generic/log"
	schedule "gitlab.mashroom.com/mashroom/backend/schedule/api"
)

// ConsumeReminderHourly handles set of hourly invocations
func ConsumeReminderHourly(
	ctx context.Context,
	req *schedule.Reminder_ConsumeRequest,
) {
	fields := log.Fields{
		"invocation_id": req.GetInvocationId(),
		"task_id":       req.GetTaskId(),
	}

	err := consumeReminderHourlyNotificationsNotResponded3hrs(ctx)
	if err != nil {
		log.WithError(err).WithFields(fields).Error("executing hourly notifications: 3 hours pending offer")
	}

	err = consumeReminderHourlyNotificationsWeekLeftToMovein(ctx)
	if err != nil {
		log.WithError(err).WithFields(fields).Error("executing hourly notifications: week to movein")
	}

	err = consumeReminderHourlyCompleteCheck(ctx)
	if err != nil {
		log.WithError(err).WithFields(fields).Error("executing hourly complete check")
	}

	err = consumeReminderHourlySynchronize(ctx)
	if err != nil {
		log.WithError(err).WithFields(fields).Error("executing hourly synchronization")
	}

	err = consumeReminderHourlyRelisting(ctx)
	if err != nil {
		log.WithError(err).WithFields(fields).Error("executing awaiting tenancy offer check")
	}
}
