package model

import (
	"gitlab.mashroom.com/mashroom/backend/common/generic/util"
	offer "gitlab.mashroom.com/mashroom/backend/offer/api"
)

// MakeVerification preparers new verification model.
func MakeVerification() *offer.OfferVerification {
	return &offer.OfferVerification{
		Id:               &offer.OfferValidation{},
		Property:         &offer.OfferValidation{},
		Viewing:          &offer.OfferValidation{},
		Contract:         &offer.OfferValidation{},
		CreatedAtValue:   &offer.OfferValidation{},
		UpdatedAtValue:   &offer.OfferValidation{},
		AcceptedAtValue:  &offer.OfferValidation{},
		DeniedAtValue:    &offer.OfferValidation{},
		CancelledAtValue: &offer.OfferValidation{},
		State:            &offer.OfferValidation{},
		Tenants:          &offer.OfferValidation{},
		Conditions: &offer.OfferConditionsValidation{
			Price:            &offer.OfferValidation{},
			DepositType:      &offer.OfferValidation{},
			TermMonths:       &offer.OfferValidation{},
			MoveinDate:       &offer.OfferValidation{},
			Pets:             &offer.OfferValidation{},
			PetsDescription:  &offer.OfferValidation{},
			Comments:         &offer.OfferValidation{},
			TenantsPitch:     &offer.OfferValidation{},
			PaymentFrequency: &offer.OfferValidation{},
			BreakClause:      &offer.OfferValidation{},
		},
		Landlord: &offer.OfferLandlordValidation{
			FirstName:      &offer.OfferValidation{},
			LastName:       &offer.OfferValidation{},
			SignatureValue: &offer.OfferValidation{},
			EmailPrimary:   &offer.OfferValidation{},
			PhonePrimary:   &offer.OfferValidation{},
			BillingAddress: &offer.OfferValidation{},
		},
		Details: &offer.OfferDetailsValidation{
			MoveinAlone:    &offer.OfferValidation{},
			AdultFlatmates: &offer.OfferValidation{},
		},
		Payment: &offer.OfferPaymentValidation{
			ReceiptId:     &offer.OfferValidation{},
			Amount:        &offer.OfferValidation{},
			Status:        &offer.OfferValidation{},
			Type:          &offer.OfferValidation{},
			CreatedAt:     &offer.OfferValidation{},
			UpdatedAt:     &offer.OfferValidation{},
			Services:      &offer.OfferValidation{},
			InvoiceNumber: &offer.OfferValidation{},
			CsrfToken:     &offer.OfferValidation{},
		},
		PaymentHistory: &offer.OfferValidation{},
		Approve: &offer.OfferApproveValidation{
			Admin:      &offer.OfferValidation{},
			Approved:   &offer.OfferValidation{},
			ApprovedAt: &offer.OfferValidation{},
		},
		Deleted: &offer.OfferValidation{},
		Log:     &offer.OfferValidation{},
		Test:    &offer.OfferValidation{},
	}
}

func verifyEmptyOffer(valid *bool, off *OfferModel, verif *offer.OfferVerification) {
	if off.Property == "" {
		*valid = false
		verif.Property.Errors = append(verif.Property.Errors, offer.OfferError_PROPERTY_EMPTY)
	}
}

func verifyConditionsOffer(valid *bool, off *OfferModel, verif *offer.OfferVerification) {
	if off.Conditions == nil {
		off.Conditions = &OfferConditionsModel{}
	}
	if util.UnPtrFloat64(off.Conditions.Price) <= 0 {
		*valid = false
		verif.Conditions.Price.Errors = append(verif.Conditions.Price.Errors, offer.OfferError_INVALID_SUM)
	}
	if off.Conditions.DepositType == offer.DepositType_UNKNOWN_DEPOSIT_TYPE {
		*valid = false
		verif.Conditions.DepositType.Errors = append(
			verif.Conditions.DepositType.Errors,
			offer.OfferError_UNKNOWN_VALUE,
		)
	}
	if util.UnPtrInt32(off.Conditions.TermMonths) <= 0 {
		*valid = false
		verif.Conditions.TermMonths.Errors = append(verif.Conditions.TermMonths.Errors, offer.OfferError_INVALID_SUM)
	}
	if util.UnPtrInt32(off.Conditions.PaymentFrequency) <= 0 {
		*valid = false
		verif.Conditions.PaymentFrequency.Errors = append(
			verif.Conditions.PaymentFrequency.Errors,
			offer.OfferError_INVALID_SUM,
		)
	}
}

func verifyLandlordOffer(valid *bool, off *OfferModel, verif *offer.OfferVerification) {
	if off.Landlord == nil {
		off.Landlord = &LandlordModel{}
	}
	if util.UnPtrString(off.Landlord.EmailPrimary) == "" {
		*valid = false
		verif.Landlord.EmailPrimary.Errors = append(verif.Landlord.EmailPrimary.Errors, offer.OfferError_EMPTY_VALUE)
	}
	if util.UnPtrString(off.Landlord.BillingAddress) == "" {
		*valid = false
		verif.Landlord.BillingAddress.Errors = append(
			verif.Landlord.BillingAddress.Errors,
			offer.OfferError_EMPTY_VALUE,
		)
	}
	if util.UnPtrString(off.Landlord.FirstName) == "" {
		*valid = false
		verif.Landlord.FirstName.Errors = append(verif.Landlord.FirstName.Errors, offer.OfferError_EMPTY_VALUE)
	}
	if util.UnPtrString(off.Landlord.LastName) == "" {
		*valid = false
		verif.Landlord.LastName.Errors = append(verif.Landlord.LastName.Errors, offer.OfferError_EMPTY_VALUE)
	}
}

func verifyContractOffer(valid *bool, off *OfferModel, verif *offer.OfferVerification) {
	if off.Contract == "" {
		*valid = false
		verif.Contract.Errors = append(verif.Contract.Errors, offer.OfferError_PROPERTY_EMPTY)
	}
}

// VerifyAll applies all verifications in required order to specified offer model.
func VerifyAll(valid *bool, model *OfferModel, verif *offer.OfferVerification) {
	verifyEmptyOffer(valid, model, verif)
	verifyConditionsOffer(valid, model, verif)
	verifyLandlordOffer(valid, model, verif)
	verifyContractOffer(valid, model, verif)
}
